==> EspoCRM-9.1.0.zip: EspoCRM-9.1.0/application/Espo/Tools/Pdf/Data/DataLoaderManager.php <==
<?php
/************************************************************************
 * This file is part of EspoCRM.
 *
 * EspoCRM – Open Source CRM application.
 * Copyright (C) 2014-2025 Yurii Kuznietsov, Taras Machyshyn, Oleksii Avramenko
 * Website: https://www.espocrm.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "EspoCRM" word.
 ************************************************************************/

namespace Espo\Tools\Pdf\Data;

use Espo\ORM\Entity;

use Espo\Core\Utils\Metadata;
use Espo\Core\InjectableFactory;

use Espo\Tools\Pdf\Data;
use Espo\Tools\Pdf\Params;

class DataLoaderManager
{
    private Metadata $metadata;
    private InjectableFactory $injectableFactory;

    public function __construct(Metadata $metadata, InjectableFactory $injectableFactory)
    {
        $this->metadata = $metadata;
        $this->injectableFactory = $injectableFactory;
    }

    public function load(Entity $entity, ?Params $params = null, ?Data $data = null): Data
    {
        if (!$params) {
            $params = Params::create();
        }

        if (!$data) {
            $data = Data::create();
        }

        /** @var class-string<DataLoader>[] $classNameList */
        $classNameList = $this->metadata->get(['pdfDefs', $entity->getEntityType(), 'dataLoaderClassNameList']) ?? [];

        foreach ($classNameList as $className) {
            $loader = $this->createLoader($className);

            $loadedData = $loader->load($entity, $params);

            $data = $data->withAdditionalTemplateData($loadedData);
        }

        return $data;
    }

    /**
     * @param class-string<DataLoader> $className
     */
    private function createLoader(string $className): DataLoader
    {
        return $this->injectableFactory->create($className);
    }
}

==> EspoCRM-9.2.7.zip: EspoCRM-9.2.7/application/Espo/Tools/Pdf/Data/DataLoaderManager.php <==
<?php
/************************************************************************
 * This file is part of EspoCRM.
 *
 * EspoCRM – Open Source CRM application.
 * Copyright (C) 2014-2025 EspoCRM, Inc.
 * Website: https://www.espocrm.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "EspoCRM" word.
 ************************************************************************/

namespace Espo\Tools\Pdf\Data;

use Espo\ORM\Entity;
use Espo\Core\Utils\Metadata;
use Espo\Core\InjectableFactory;
use Espo\Tools\Pdf\AttachmentProvider;
use Espo\Tools\Pdf\Data;
use Espo\Tools\Pdf\Params;

class DataLoaderManager
{
    public function __construct(
        private Metadata $metadata,
        private InjectableFactory $injectableFactory,
    ) {}

    public function load(Entity $entity, ?Params $params = null, ?Data $data = null): Data
    {
        if (!$params) {
            $params = Params::create();
        }

        if (!$data) {
            $data = Data::create();
        }

        $defs = $this->metadata->get("pdfDefs.{$entity->getEntityType()}") ?? [];

        /** @var class-string<DataLoader>[] $loaderClassList */
        $loaderClassList = $defs['dataLoaderClassNameList'] ?? [];

        foreach ($loaderClassList as $className) {
            $loadedData = $this->createLoader($className)
                ->load($entity, $params);

            $data = $data->withAdditionalTemplateData($loadedData);
        }

        /** @var class-string<AttachmentProvider<Entity>>[] $attachmentProviderClassList */
        $attachmentProviderClassList = $defs['attachmentProviderClassNameList'] ?? [];

        foreach ($attachmentProviderClassList as $className) {
            $provider = $this->createProvider($className);

            $attachments = $provider->get($entity, $params);

            $data = $data->withAttachmentsAdded($attachments);
        }

        return $data;
    }

    /**
     * @param class-string<DataLoader> $className
     */
    private function createLoader(string $className): DataLoader
    {
        return $this->injectableFactory->create($className);
    }

    /**
     * @param class-string<AttachmentProvider<Entity>> $className
     * @return AttachmentProvider<Entity>
     */
    private function createProvider(string $className): AttachmentProvider
    {
        /** @var AttachmentProvider<Entity> */
        return $this->injectableFactory->create($className);
    }
}
